@@ -171,15 +171,29 @@ class PBTableTab(QtGui.QWidget):
         self.toolbar = LayoutWidget(layout, QtGui.QToolBar(self))
         # build the table and attach a model
         self.table = LayoutWidget(layout,TableView(extensionId=suffixExtId(self, "Table")))
-        q=self.query
-        if self.filters is not None:
-            for col, value in self.filters:
-                q = q.filter(col==value)
-                self.entity.info[colAttr(col)] = value
-        self.table.setModel(QueryTableModel(q, ssrc=ssrc, objectConverter=self.objectConverter), 
+        self.table.setModel(QueryTableModel(self.query, ssrc=ssrc, objectConverter=self.objectConverter), 
                             extensionId=suffixExtId(self, "Table"))
 
         self.bindings = self.entity.itemCommands.withView(self.table, bindDefault=True)
         self.bindings.fillToolbar(self.toolbar)
 
+        dataContextNeeded = False
+        if self.filters is not None:
+            for col, value in self.filters:
+                if callable(value):
+                    dataContextNeeded = True
+
+        if not dataContextNeeded:
+            self.refresh()
+
+    def refresh(self, dataContext=None):
+        q=self.query
+        if self.filters is not None:
+            for col, value in self.filters:
+                if callable(value):
+                    value = value(dataContext)
+                q = q.filter(col==value)
+                self.entity.info[colAttr(col)] = value
+        self.table.model().query = q
+
         self.table.model().reset_content_from_session()
