@@ -13,19 +13,33 @@ do this here as an example and to prepare for the doc-tests.  Note that it is no
 illegal to call sip.setapi twice, but the second call must agree in api version with
 the first.
 
-    >>> import sip
-    >>> sip.setapi('QString', 2)
-    >>> sip.setapi('QVariant', 2)
+    >>> if qt_bindings == 'PyQt4':
+    ...     import sip
+    ...     sip.setapi('QString', 2)
+    ...     sip.setapi('QVariant', 2)
 """
 
-import sip
-sip.setapi('QString', 2)
-sip.setapi('QVariant', 2)
+qt_bindings = 'PyQt4'
+if qt_bindings == 'PyQt4':
+    # we wish to only touch sip if we're PyQt4 based (otherwise sip is unnecessary)
+    import sip
+    sip.setapi('QString', 2)
+    sip.setapi('QVariant', 2)
 from PyQt4 import QtGui,QtCore
 import datetime
 import decimal
 import re
 
+# with this conditional, we abstract away this difference between pyqt and pyside
+if QtCore.__name__.startswith("PySide"):
+    Slot = QtCore.Slot
+    Signal = QtCore.Signal
+    Property = QtCore.Property
+else:
+    Slot = QtCore.pyqtSlot
+    Signal = QtCore.pyqtSignal
+    Property = QtCore.pyqtProperty
+
 def fromQType(v, suggested=None):
     """
     This function takes a PyQt/PySide type and returns the most faithful representative of the item 
@@ -200,6 +214,23 @@ def suffixExtId(parent, ext):
     else:
         return id+"/"+ext
 
+class TabMenuHolder:
+    def __init__(self, tab):
+        self.tab = tab
+
+        self.actSaveLast = QtGui.QAction("&Reopen to last viewed tab", self.tab)
+        self.actSaveLast.setCheckable(True)
+        self.actSaveThis = QtGui.QAction("&Reopen to current tab", self.tab)
+        self.actSaveThis.setCheckable(True)
+
+        self.tab.setContextMenuPolicy(QtCore.Qt.ActionsContextMenu)
+        self.tab.addAction(self.actSaveLast)
+        self.tab.addAction(self.actSaveThis)
+
+    def setChecks(self, status):
+        self.actSaveLast.setChecked(status == WindowGeometry.TAB_REOPEN_LAST)
+        self.actSaveThis.setChecked(status == WindowGeometry.TAB_REOPEN_SPECIFIED)
+
 class WindowGeometry(QtCore.QObject):
     """
     This class saves and restores the size and other geometry artifacts about 
@@ -218,13 +249,23 @@ class WindowGeometry(QtCore.QObject):
     * widget.property("ExtensionId")
     * widget.objectName()  (recommended)
 
+    The QTabWidgets in the `tabs` list will have a context menu added on them 
+    with options about how the tab selection should be recalled on reload.  They 
+    can reopen the last opened, always reopen to a specific tab, or let it as 
+    default (probably meaning the first tab is shown).
+
     :param widget: the QWidget for which to save & restore state
     :param name: optional identifier to associate this in the persistent state
     :param size: save & restore window size (default True)
     :param position: save & restore window position (default True)
-    :param splitters: splitters to save position
+    :param splitters: list of splitters to save position
+    :param tabs: list of QTabWidgets whose tab selection should be managed/remembered
     """
-    def __init__(self, widget, name=None, size=True, position=True, splitters=None):
+    TAB_REOPEN_DEFAULT = 0
+    TAB_REOPEN_LAST = 1
+    TAB_REOPEN_SPECIFIED = 2
+
+    def __init__(self, widget, name=None, size=True, position=True, splitters=None, tabs=None):
         QtCore.QObject.__init__(self, widget)
 
         self.widget = widget
@@ -236,10 +277,18 @@ class WindowGeometry(QtCore.QObject):
         if self.name is None:
             self.name = widget.objectName()
         self.splitters = splitters if splitters else []
+        self.tabs = tabs if tabs else []
+        self.tabs = [TabMenuHolder(t) for t in self.tabs]
 
         for s_index in range(len(self.splitters)):
             self.splitters[s_index].splitterMoved.connect(lambda pos,index,x=s_index: self.updateSplitter(x,pos,index))
 
+        self.tabActions = []
+        for t_index in range(len(self.tabs)):
+            self.tabs[t_index].tab.currentChanged.connect(lambda newSel,x = t_index: self.updateTab(x,newSel))
+            self.tabs[t_index].actSaveLast.triggered.connect(lambda checked, kind=self.TAB_REOPEN_LAST, x=t_index: self.updateTabLastSave(x, checked, kind))
+            self.tabs[t_index].actSaveThis.triggered.connect(lambda checked, kind=self.TAB_REOPEN_SPECIFIED, x=t_index: self.updateTabLastSave(x, checked, kind))
+
         self.restoreState()
 
         if hasattr(self.widget, "finished"):
@@ -247,22 +296,24 @@ class WindowGeometry(QtCore.QObject):
         else:
             self.widget.installEventFilter(self)
 
-    def finished(self,i):
-        self.saveState(splitters=False)
+    def finished(self, i):
+        self.saveState(splitters=False, tabs=False)
 
     def eventFilter(self, obj, event):
         if obj is self.widget and event.type() == QtCore.QEvent.Close:
-            self.saveState(splitters=False)
+            self.saveState(splitters=False, tabs=False)
         return QtCore.QObject.eventFilter(self, obj, event)
 
-    def saveState(self, splitters=True):
+    def saveState(self, splitters=True, tabs=True):
         """
         This saves the state of all controlled elements.  Some elements are 
         saved immediately when modified (such as splitters).  Thus we suppress 
         the state saving on close for these elements.
-        
+
         :param splitters:  pass False to suppress the saving of the splitter 
             state for splitters passed in __init__
+        :param tabs:  pass False to suppress the saving of the tab 
+            state for tabs passed in __init__
         """
         settings = QtCore.QSettings()
         settings.beginGroup(self.name)
@@ -281,6 +332,10 @@ class WindowGeometry(QtCore.QObject):
             for splitter_index in range(len(self.splitters)):
                 settings.setValue(self.splitter_persist_location(splitter_index),self.splitters[splitter_index].saveState())
 
+        if tabs:
+            for tab_index in range(len(self.tabs)):
+                settings.setValue(self.tab_persist_location(tab_index, "to-reopen"), self.tabs[tab_index].tab.currentIndex())
+
         settings.endGroup()
 
     def restoreState(self):
@@ -305,6 +360,13 @@ class WindowGeometry(QtCore.QObject):
             state = settings.value(self.splitter_persist_location(splitter_index))
             if state is not None:
                 self.splitters[splitter_index].restoreState(state)
+
+        for tab_index in range(len(self.tabs)):
+            reopen = int(settings.value(self.tab_persist_location(tab_index, "reopen-status"), self.TAB_REOPEN_LAST))
+            self.tabs[tab_index].setChecks(reopen)
+            state = settings.value(self.tab_persist_location(tab_index, "to-reopen"))
+            if state is not None and reopen != self.TAB_REOPEN_DEFAULT:
+                self.tabs[tab_index].tab.setCurrentIndex(int(state))
         settings.endGroup()
 
     def splitter_persist_location(self, splitter_index):
@@ -313,12 +375,44 @@ class WindowGeometry(QtCore.QObject):
             splitter_name = str(splitter_index)
         return "splitter/{0}".format(splitter_name)
 
-    def updateSplitter(self,splitter_index,pos,index):
+    def updateSplitter(self, splitter_index, pos, index):
         settings = QtCore.QSettings()
         settings.beginGroup(self.name)
         settings.setValue(self.splitter_persist_location(splitter_index),self.splitters[splitter_index].saveState())
         settings.endGroup()
 
+    def tab_persist_location(self, tab_index, v):
+        tab_name = self.tabs[tab_index].tab.objectName()
+        if tab_name in [None, ""]:
+            tab_name = str(tab_index)
+        return "tab/{0}/{1}".format(tab_name, v)
+
+    def updateTabLastSave(self, tab_index, checked, how):
+        reopen = None
+        tab_to_reopen = None
+        if checked:
+            reopen = how
+            tab_to_reopen = self.tabs[tab_index].tab.currentIndex()
+        else:
+            reopen = self.TAB_REOPEN_DEFAULT
+
+        if reopen is not None:
+            self.tabs[tab_index].setChecks(reopen)
+
+            settings = QtCore.QSettings()
+            settings.beginGroup(self.name)
+            settings.setValue(self.tab_persist_location(tab_index, "reopen-status"), reopen)
+            if tab_to_reopen is not None:
+                settings.setValue(self.tab_persist_location(tab_index, "to-repen"), tab_to_reopen)
+            settings.endGroup()
+
+    def updateTab(self, tab_index, newSel):
+        settings = QtCore.QSettings()
+        settings.beginGroup(self.name)
+        if int(settings.value(self.tab_persist_location(tab_index, "reopen-status"), self.TAB_REOPEN_LAST)) == self.TAB_REOPEN_LAST:
+            settings.setValue(self.tab_persist_location(tab_index, "to-reopen"), self.tabs[tab_index].tab.currentIndex())
+        settings.endGroup()
+
 
 class OnlineHelp(QtCore.QObject):
     """
