@@ -19,7 +19,6 @@ from sqlalchemy_helper import *
 import sqlalchemy
 import datetime
 import decimal
-from qtalchemy.widgets import PBDateEdit
 from user_attr import UserAttr, AttrNumeric
 
 if sqlalchemy.__version__.split('.') < ['0', '6', '0']:
@@ -347,7 +346,7 @@ class AlchemyModelDelegate(QtGui.QStyledItemDelegate):
                 c = index.model().columns[index.column()]
                 userattr = getattr(cls,c.attr if hasattr(c,"attr") else c)
                 if userattr is not None and hasattr(userattr,"WidgetBind"):
-                    userattr.WidgetBind(editor,index.model().row_object(index.row()))
+                    userattr.WidgetBind(editor,index.model().row_object_by_index(index))
         except AttributeError, e:
             pass
 
@@ -386,6 +385,7 @@ def AttrWidgetFactory(userattr,parent=None, index=None):
         return userattr.WidgetFactory(parent, index)
     myAttrType = ClassAttributeType(userattr)
     if myAttrType == datetime.date:
+        from qtalchemy.widgets import PBDateEdit
         w = PBDateEdit(parent)
         #w = QtGui.QDateEdit(parent)
         #w.setCalendarPopup(True)
@@ -403,6 +403,7 @@ def AttrWidgetFactory(userattr,parent=None, index=None):
     else:
         return QtGui.QLineEdit(parent)
 
+
 class ModelColumn:
     def __init__(self,attr,type_,label=None,readonly=False):
         self.attr = attr
@@ -414,6 +415,33 @@ class ModelColumn:
         self.readonly = readonly
 
 
+class ObjectRepresent(object):
+    def __init__(self, idattr, idListMimeType=None):
+        """
+        Instances of this class can be used for an objectConverter for a 
+        PBTableModel.  Such instances take care of resolving the primary key 
+        for passing to a CommandEvent.  They also fill a QMimeData with a list 
+        of the primary keys in a set of model indexes.
+        
+        A model exposing mime data in this way can be used as a source of items 
+        draggable with actions on drop defined by a function decorated with 
+        :func:`CommandMenu.dropCommand`.
+        
+        :param idattr:  The attribute which holds the primary key of this object
+        :param idListMimeType:  The mime type for a list of id's.
+        """
+        self.idattr = idattr
+        self.idListMimeType = idListMimeType
+        
+    def __call__(self, obj):
+        return getattr(obj, self.idattr)
+
+    def mime(self, qmime, model, indexes):
+        if self.idListMimeType is not None:
+            ids = set([self(x.internalPointer()) for x in indexes])
+            qmime.setData(self.idListMimeType, '\n'.join([str(x) for x in ids]))
+
+
 class SACollectionManagerForModel:
     """
     Provide semi-hookable functions from PBTableModel for adding and deleting
@@ -485,7 +513,7 @@ class SACollectionManagerForModel:
         except Exception as e:
             return False
 
-def ClassTableModel(cls, attributes, readonly=True, fixed_rows=False, row_manager=None, objectConverter=None):
+def ClassTableModel(cls, attributes, readonly=True, fixed_rows=False, row_manager=None, objectConverter=None, tree_structure=None):
     """
     Create a :class:`PBTableModel` exposing a list of objects of a particular python class.
     
@@ -548,10 +576,10 @@ def ClassTableModel(cls, attributes, readonly=True, fixed_rows=False, row_manage
         ua = getattr(cls, attr)
         return ModelColumn(attr, ClassAttributeType(ua), ClassAttributeLabel(ua), readonly=attrReadonly(cls, attr))
     cols = [makeModelColumn(cls, a) for a in attributes]
-    return PBTableModel(cols, cls=cls, readonly=readonly, fixed_rows=fixed_rows, row_manager=row_manager, objectConverter=objectConverter)
+    return PBTableModel(cols, cls=cls, readonly=readonly, fixed_rows=fixed_rows, row_manager=row_manager, objectConverter=objectConverter, tree_structure=tree_structure)
 
 
-def QueryClassTableModel(cls, attributes, query, ssrc, objectConverter=None):
+def QueryClassTableModel(cls, attributes, query, ssrc, objectConverter=None, tree_structure=None):
     """
     Construct a :class:`PBTableModel` from a sqlalchemy query returning a mapped 
     class.
@@ -566,9 +594,9 @@ def QueryClassTableModel(cls, attributes, query, ssrc, objectConverter=None):
         ua = getattr(cls, attr)
         return ModelColumn(attr, ClassAttributeType(ua), ClassAttributeLabel(ua), readonly=attrReadonly(cls, attr))
     cols = [makeModelColumn(cls, a) for a in attributes]
-    return PBTableModel(cols, query=query, cls=cls, ssrc=ssrc, objectConverter=objectConverter)
+    return PBTableModel(cols, query=query, cls=cls, ssrc=ssrc, objectConverter=objectConverter, tree_structure=tree_structure)
 
-def QueryTableModel(q, cols=None, include_uuid=False, ssrc=None, objectConverter=None):
+def QueryTableModel(q, cols=None, include_uuid=False, ssrc=None, objectConverter=None, tree_structure=None):
     """
     This helper function constructs an :class:`PBTableModel` from a sqlalchemy query.  It can
     auto-detect the columns and types or take a list of column names from the query.
@@ -603,15 +631,15 @@ def QueryTableModel(q, cols=None, include_uuid=False, ssrc=None, objectConverter
         objectConverter = lambda x, col=uuid_cols[0]: getattr(x, col.name)
 
     if q.session is None:
-        return PBTableModel(model_cols, query=q, ssrc=ssrc, objectConverter=objectConverter)
+        return PBTableModel(model_cols, query=q, ssrc=ssrc, objectConverter=objectConverter, tree_structure=tree_structure)
     else:
-        return PBTableModel(model_cols, rows=q.all(), objectConverter=objectConverter)
+        return PBTableModel(model_cols, rows=q.all(), objectConverter=objectConverter, tree_structure=tree_structure)
 
-class PBTableModel(QtCore.QAbstractTableModel):
+class PBTableModel(QtCore.QAbstractItemModel):
     """
     A PyQt/PySide model for objects with :class:`UserAttr`.
     """
-    def __init__(self, columns, cls=None, rows=None, query=None, ssrc=None, parent=None, readonly=True, row_manager=None, fixed_rows=False, objectConverter=None):
+    def __init__(self, columns, cls=None, rows=None, query=None, ssrc=None, parent=None, readonly=True, row_manager=None, fixed_rows=False, objectConverter=None, tree_structure=None):
         """
         All parameters other than columns are optional.
 
@@ -621,7 +649,7 @@ class PBTableModel(QtCore.QAbstractTableModel):
         :param ssrc:  A sqlalchemy sessionmaker.
         :param objectConverter: Callable taking row objects to unique keys in the application domain
         """
-        QtCore.QAbstractTableModel.__init__(self,parent)
+        QtCore.QAbstractItemModel.__init__(self,parent)
         self.columns = columns
         self.cls = cls
 
@@ -635,6 +663,10 @@ class PBTableModel(QtCore.QAbstractTableModel):
         if row_manager is None:
             row_manager = SACollectionManagerForModel(self.cls, fixed_rows=fixed_rows)
         self.row_manager = row_manager
+        
+        # dropCallable is a mapping of mime-types to callable functions
+        self.dropCallable = {}
+        self.tree_structure = tree_structure
 
         self.setRoleNames(dict([(QtCore.Qt.UserRole+i,self.columns[i].attr) for i in range(len(columns))]))
 
@@ -683,12 +715,32 @@ class PBTableModel(QtCore.QAbstractTableModel):
         q2.session.close()
         q2.session = None
 
+    def objParent(self, rowObj):
+        if self.tree_structure is None:
+            return None
+        elif rowObj is None:
+            return None
+        else:
+            return getattr(rowObj, self.tree_structure[0])
+
+    def objChildren(self, rowObj):
+        if self.tree_structure is None:
+            return None
+        else:
+            return getattr(rowObj, self.tree_structure[1])
+
     def rowReverseIndex(self, row_obj):
-        for i in range(self.rowCount(None)):
-            row = self.row_object(i)
-            if row is row_obj:
-                return i
-        return -1
+        if self.objParent(row_obj) is not None:
+            row_group = self.objChildren(self.objParent(row_obj))
+            for i in range(len(row_group)):
+                if row_group[i] == row_obj:
+                    return self.createIndex(i, 0, row_obj)
+        else:
+            for i in range(self.rowCount(QtCore.QModelIndex())):
+                row = self.row_object(i, None)
+                if row is row_obj:
+                    return self.createIndex(i, 0, row_obj)
+        return QtCore.QModelIndex()
 
     def rowEmitChange(self, row_obj, col):
         """
@@ -703,8 +755,8 @@ class PBTableModel(QtCore.QAbstractTableModel):
         else:
             col1 = col2 = col
         i = self.rowReverseIndex(row_obj)
-        if i != -1:
-            self.dataChanged.emit(self.index(i,col1,None),self.index(i,col2,None))
+        if i.isValid():
+            self.dataChanged.emit(self.index(i.row(),col1,i.parent()),self.index(i.row(),col2,i.parent()))
 
     def row_object_change(self, row_obj, key, oldvalue):
         col_changed = -1
@@ -716,31 +768,53 @@ class PBTableModel(QtCore.QAbstractTableModel):
             return  # if the changing key is not a member of this model, then nothing to do here
         self.rowEmitChange(row_obj, col_changed)
 
-    def row_object(self,row):
-        if self.row_list is None or row < 0:
+    def row_object_by_index(self, index):
+        return index.internalPointer()
+
+    def row_object(self, row, parent):
+        if parent is None:
+            row_group = self.row_list
+        else:
+            row_group = self.objChildren(parent)
+
+        if row_group is None or row < 0:
             return None
 
-        if row < len(self.row_list):
-            return self.row_list[row]
+        if row < len(row_group):
+            return row_group[row]
         else:
-            return self.flipper_rows[row - len(self.row_list)]
+            return self.flipper_rows[row - len(row_group)]
 
-    def index(self,row,column,parent):
-        return self.createIndex(row,column,self.row_object(row))
+    def index(self, row, column, parent):
+        if parent is None:
+            parent = QtCore.QModelIndex()
+        return self.createIndex(row, column, self.row_object(row, parent.internalPointer()))
+
+    def parent(self, child):
+        obj = child.internalPointer()
+        parent = self.objParent(obj)
+        if parent is None:
+            return QtCore.QModelIndex()
+        else:
+            return self.rowReverseIndex(parent)
 
     def columnCount(self,parent):
         #print "columnCount", len(self.columns)
         return len(self.columns)
 
-    def rowCount(self,parent):
+    def rowCount(self, parent):
         if self.row_list is None:
             return 0
 
         #print "rowCount", len(self.row_list)
-        if self.row_manager.fixed_rows:
-            return len(self.row_list)
+        if self.tree_structure is None or parent is None or not parent.isValid():
+            if self.row_manager.fixed_rows:
+                return len(self.row_list)
+            else:
+                return len(self.row_list) + len(self.flipper_rows)
         else:
-            return len(self.row_list) + len(self.flipper_rows)
+            obj = parent.internalPointer()
+            return len(self.objChildren(obj))
 
     def removeRows(self,row,count,parent):
         """
@@ -752,7 +826,7 @@ class PBTableModel(QtCore.QAbstractTableModel):
         else:
             self.beginRemoveRows(parent,row,row+count-1)
             for i in range(count):
-                row_obj = self.row_object(row)
+                row_obj = self.row_object(row, parent.internalPointer())
                 if row_obj in self.row_list:
                     self.row_manager.delete_row(row_obj)
             self.endRemoveRows()
@@ -765,7 +839,7 @@ class PBTableModel(QtCore.QAbstractTableModel):
 
     def data(self, index, role):
         c = self.columns[index.column()]
-        r = self.row_object(index.row())
+        r = self.row_object_by_index(index)
         if not index.isValid():
             return None
 
@@ -806,10 +880,10 @@ class PBTableModel(QtCore.QAbstractTableModel):
         return None
 
     def ensureNoFlipper(self, index):
-        if index.row() >= len(self.row_list):
+        if not self.row_manager.fixed_rows and index.row() >= len(self.row_list):
             self.row_manager.join_flipper(self.flipper_rows[0])
             del self.flipper_rows[0]
-            if not self.row_manager.fixed_rows and len(self.flipper_rows) == 0:
+            if len(self.flipper_rows) == 0:
                 self.beginInsertRows(QtCore.QModelIndex(),len(self.row_list),len(self.row_list))
                 self.flipper_rows.append(self.row_manager.add_flipper_row())
                 self.endInsertRows()
@@ -822,7 +896,7 @@ class PBTableModel(QtCore.QAbstractTableModel):
             #return None
 
         c = self.columns[index.column()]
-        r = self.row_object(index.row())
+        r = self.row_object_by_index(index)
         newValue = fromQType(value, attrType(self.cls, c))
         oldValue = getattr(r, c.attr if hasattr(c, "attr") else c)
         #print attrType(self.cls, c), newValue, oldValue
@@ -843,8 +917,74 @@ class PBTableModel(QtCore.QAbstractTableModel):
                 result |= QtCore.Qt.ItemIsUserCheckable
             else:
                 result |= QtCore.Qt.ItemIsEditable
+        result |= QtCore.Qt.ItemIsDragEnabled
+        if len(self.dropCallable) > 0:
+            result |= QtCore.Qt.ItemIsDropEnabled
         return result
 
+    def supportedDropActions(self):
+        return QtCore.Qt.ActionMask
+
+    def dropMimeData(self, data, action, row, column, parent):
+        x = False
+        for type in data.formats():
+            if self.dropCallable.has_key(type):
+                x = self.dropCallable[type](data, action, row, column, parent)
+        return x
+
+    def mimeTypes(self):
+        return self.dropCallable.keys() + ["text/plain"]
+
+    def mimeData(self, indexes):
+        result = modelMimeRectangle(self, indexes)
+        if self.objectConverter is not None and hasattr(self.objectConverter, "mime"):
+            self.objectConverter.mime(result, self, indexes)
+        return result
+
+
+def modelMimeRectangle(model, selected):
+    """
+    Fill and return a QMimeData with the text in the indexes listed.  The copied 
+    data is a rectangular array of the selected cells.  Cells not selected that 
+    fall with-in a row or column of the enclosing rectangle, are indicated only by 
+    an empty string in the tab delimited grid.
+    """
+    def index_row_lineage(index):
+        r = []
+        while index.isValid():
+            r.append(index.row())
+            index = index.parent()
+        r.reverse()
+        return tuple(r)
+
+    def row_data(full_cols, my_indexes):
+        result = []
+        my_cols = dict([(i.column(), i) for i in my_indexes])
+        for x in full_cols:
+            r = ""
+            if x in my_cols:
+                r = str(fromQType(model.data(my_cols[x], QtCore.Qt.DisplayRole)))
+            result.append(r)
+        return result
+
+    m = QtCore.QMimeData()
+    rows = []
+    if len(selected):
+        sortable_index_list = [(index_row_lineage(i), i.column(), i) for i in selected]
+        cols = list(set([x[1] for x in sortable_index_list]))
+        cols.sort()
+
+        rows = list(set([x[0] for x in sortable_index_list]))
+        rows.sort()
+
+        textrows = ['\t'.join([model.headerData(x,QtCore.Qt.Horizontal,QtCore.Qt.DisplayRole) for x in cols])]
+        for y in rows:
+            my_indexes = [x[2] for x in sortable_index_list if x[0] == y]
+            textrows.append('\t'.join(row_data(cols, my_indexes)))
+    m.setData("text/plain",'\n'.join(textrows))
+    return m
+
+
 class WidgetAttributeMapper(object):
     """
     This class provides a hook between widgets in the graphical toolkit and 
@@ -889,6 +1029,31 @@ class WidgetAttributeMapper(object):
             self.attribute_map[attr] = []
         self.attribute_map[attr].append(yoke)
 
+    def focusAttribute(self, attr):
+        """
+        Set focus the input yoke corresponding to the attribute attr.
+        
+        If attr is not a mapped attribute, this method does nothing.
+        """
+        if attr in self.attribute_map:
+            y = self.attribute_map[attr]
+            if len(y) > 0:
+                y[0].focus()
+
+    def attributeWidget(self, attr):
+        """
+        Returns the main qt widget in the yoke corresponding to attr.  The 
+        exact meaning of the return of this function is subject to the 
+        yoke implementation.
+        
+        If attr is not a mapped attribute, this method returns None.
+        """
+        if attr in self.attribute_map:
+            y = self.attribute_map[attr]
+            if len(y) > 0:
+                return y[0].mainWidget()
+        return None
+
     def getObjectAttr(self,attr):
         return getattr(self.obj,attr)
 
@@ -1002,17 +1167,37 @@ class WidgetAttributeMapper(object):
 
 class MapperMixin(object):
     """
-    :class:`MapperMixin` holds a list of :class:`WidgetAttributeMapper` classes and 
+    MapperMixin holds a list of :class:`WidgetAttributeMapper` classes and 
     provides helpers to ensure that the whole collection is synchronized before save.
     """
     def init_mapper_list(self):
         if not hasattr(self,"_mapper_list"):
             self._mapper_list = []
 
-    def mapClass(self, cls, objectConverter=None):
+    def mapClass(self, cls, loaderTarget=None, objectConverter=None):
+        """
+        This function creates a new :class:`WidgetAttributeMapper`, adds it 
+        to the list of mappers and returns this new instance.
+        
+        In addition, if loaderTarget is specified, a callable loader function is
+        constructed and passed to self.addLoader.  It is assumed that we are
+        mixed in with some class that has loader callbacks.  Such an
+        implemenation is :class:`qtalchemy.dialogs.BoundDialog`.
+        
+        :param loaderTarget:  a callable taking the dataContext of the ambient
+            dialog and returning an object instance to connect to the return
+            WidgetAttributeMapper
+        :param objectConverter:  a callable converting the object bound to the
+            WidgetAttributeMapper to an identifier primary key.
+        """
         self.init_mapper_list()
         m = WidgetAttributeMapper(cls, objectConverter=objectConverter)
         self._mapper_list.append(m)
+        
+        if loaderTarget is not None:
+            # this presumes we're in a BoundDialog (or something that quacks like one)
+            self.addLoader(lambda dataContext: m.connect_instance(loaderTarget(dataContext)))
+        
         return m
 
     def mapper(self,obj):
