@@ -16,14 +16,15 @@ sqlalchemy and PyQt/PySide.
 from PyQtHelpers import LayoutWidget, LayoutLayout, ButtonBoxButton, FormRow, \
             WindowGeometry, OnlineHelp, \
             writeTableColumnGeo, readTableColumnGeo, suffixExtId, \
-            fromQType, toQType, qtapp
+            fromQType, toQType, qtapp, \
+            Signal, Slot, Property
 
 from commands import DomainEntity, Command, CommandMenu, CommandEvent, \
         BoundCommandMenu, BoundCommandMenuItem
 
 from user_attr import UserAttr, Nullable, AttrNumeric, UseCachedValue
 from sqlalchemy_helper import ModelObject, ModelSession, PBSessionMaker, UUID, \
-        ValidationError, user_message, Message, instanceEvent
+        ValidationError, user_message, Message, instanceEvent, sessionExtension
 
 from input_yoke import \
     InputYoke, \
@@ -35,7 +36,8 @@ from input_yoke import \
 
 # model elements
 from PyQtModels import ClassTableModel, QueryTableModel, QueryClassTableModel, \
-        ModelColumn, PBTableModel, MapperMixin, AlchemyModelDelegate, \
+        ModelColumn, PBTableModel, AlchemyModelDelegate, modelMimeRectangle, \
+        MapperMixin, WidgetAttributeMapper, ObjectRepresent, \
         attrLabel, ClassAttributeLabel, attrType, ClassAttributeType, attrReadonly
 
 from foreign_key import ForeignKeyReferral, ForeignKeyEditYoke, ForeignKeyComboYoke
