@@ -17,10 +17,7 @@ import inspect
 from PyQtHelpers import *
 from PyQt4 import QtCore, QtGui
 
-if QtCore.__name__.startswith("PySide"):
-    Signal = QtCore.Signal
-else:
-    Signal = QtCore.pyqtSignal
+DROP_ACTION_TYPE="drop-action"
 
 class Command(object):
     """
@@ -30,7 +27,7 @@ class Command(object):
 
     See :class:`BoundCommandMenu` for more details.
     """
-    def __init__(self, descr, default=False, viewRelated=False, requireSelection=True, needs_reload=True, key=None, statusTip=None, iconFile=None, type="action", sort=500):
+    def __init__(self, descr, default=False, viewRelated=False, requireSelection=True, needs_reload=True, key=None, statusTip=None, iconFile=None, type="action", sort=500, mimeType=None):
         """
         :param descr: Text description of command for menu items
         :param default: Default command for item activation.
@@ -58,7 +55,14 @@ class Command(object):
         self.func = func
         return func
 
+    def name(self):
+        return self.func.func_name
+
     def action(self, parent):
+        # drag & drop operations are not prompted by QAction gui events (and they must come with mime data)
+        if self.command_type == DROP_ACTION_TYPE:
+            return None
+
         action = QtGui.QAction(self.descr, parent)
         if self.key is not None:
             action.setShortcut(self.key)
@@ -184,6 +188,18 @@ class CommandMenu(object):
         kwargs["type"] = "action"
         return self.command(descr, **kwargs)
 
+    def dropCommand(self, mimeType, **kwargs):
+        """
+        A command function for when a bit of mime-data is dropped on 
+        to the row.  This is typically used with-in the context of a 
+        QAbstractItemView derived widget.
+        """
+        if not kwargs.has_key("requireSelection"):
+            kwargs["requireSelection"] = True
+        kwargs["viewRelated"] = True
+        kwargs["type"] = DROP_ACTION_TYPE
+        return self.command(mimeType, **kwargs)
+
     def __get__(self, instance, owner):
         try:
             return getattr(instance, self.name)
@@ -245,6 +261,10 @@ class CommandEvent(object):
     Note that the free-form nature of screen context can create a tight coupling 
     between the screens using the associate :class:`CommandMenu` and the the 
     command functions themselves.
+    
+    This class works can work in tight conjunction with :class:`qtalchemy.dialogs.BoundDialog`.  
+    In this context, the preCommandSave function will set a member savedDataContext
+    with the current main_row object bound to the BoundDialog.
     """
     def __init__(self, cmd=None):
         self.cmd = cmd
@@ -287,6 +307,9 @@ class BoundCommandMenuItem(object):
             self.item_sel.selectionChanged.connect(self.enable_buttons)
             self.model.modelReset.connect(self.enable_buttons)
 
+            if self.cmd.command_type == DROP_ACTION_TYPE:
+                self.model.dropCallable[self.cmd.descr] = self.dropAction
+
         if isinstance(self.action, QtGui.QPushButton):
             self.action.clicked.connect(self.triggeredProxy)
         elif isinstance(self.action, QtGui.QAction):
@@ -294,10 +317,32 @@ class BoundCommandMenuItem(object):
 
     def enable_buttons(self, sel=None, desel=None):
         # setEnabled is sufficient for both QPushButton and QAction
-        if self.cmd.requireSelection:
+        if self.cmd.requireSelection and self.action is not None:
             indexes = self.binding.view.selectedIndexes()
             self.action.setEnabled(len(indexes)>0)
 
+    def dropAction(self, data, action, row, column, parent):
+        event = CommandEvent(self.cmd)
+        self.binding.preCommand.emit(event)
+        if event.aborted:
+            return
+        
+        # this is a bit of a hack to preCommand first:
+        # it is presumed that preCommand saves the data so that the database keys are up-to-date
+        handle = parent.internalPointer()
+        if handle is not None:
+            handle = self.model.objectConverter(handle)
+        args = inspect.getargspec(self.cmd.func).args
+        d = {}
+        if 'event' in args:
+            d['event'] = event
+        self.cmd.func(self.binding.target, handle, str(data.data(self.cmd.descr)), **d)
+        self.binding.refresh.emit(event)
+        if self.cmd.needs_reload:
+            if self.binding.view:
+                self.model.reset_content_from_session()
+        return not event.aborted
+
     def objectActivatedProxy(self, obj=None, objectConverter=None):
         # self.objectConverter takes precedence!
         objectConverter = objectConverter if self.objectConverter is None else self.objectConverter
@@ -373,17 +418,25 @@ class BoundCommandMenu(QtCore.QObject):
     preCommand = Signal(object, name='preCommand')
     refresh = Signal(object, name='refresh')
 
-    def __init__(self, target, menu, view=None, mapper=None, obj=None, buttonBox=None, bindDefault=False, objectConverter=None):
+    def __init__(self, target, menu, view=None, mapper=None, obj=None, webView=None, javaScriptName=None, buttonBox=None, bindDefault=False, objectConverter=None):
         QtCore.QObject.__init__(self)
         self.target = target
         self.menu = menu
         self.view = view
         self.mapper = mapper
         self.obj = obj
+        self.webView = webView
+        self.javaScriptName = javaScriptName
         self.buttonBox = buttonBox
         self.bindDefault = bindDefault
         self.objectConverter = objectConverter
         
+        if self.webView:
+            # Is loadFinished soon enough?  What if the user is clicking 
+            # links and icons before the load is finished?
+            # Note that loadStarted was too soon and the frame was overwritten.
+            self.webView.loadFinished.connect(lambda x: self.addJavascriptObjects())
+        
         if self.view:
             # we need to hook up to the context menu immediately
             self.resetActions()
@@ -394,6 +447,7 @@ class BoundCommandMenu(QtCore.QObject):
         QAction objects will enable and disable as appropriate for the connected view.
         
         :param view: a QAbstractItemView attached to a QAbstractItemModel
+        :param bindDefault: bindDefault can take several values - True, "double", "activated", False (True maps to "activated")
         """
         return BoundCommandMenu(self.target, self.menu, view=view, buttonBox=buttonBox, bindDefault=bindDefault, objectConverter=objectConverter)
 
@@ -406,6 +460,25 @@ class BoundCommandMenu(QtCore.QObject):
         """
         return BoundCommandMenu(self.target, self.menu, mapper=mapper, buttonBox=buttonBox, bindDefault=bindDefault, objectConverter=objectConverter)
 
+    def withWebView(self, webView, javaScriptName=None, objectConverter=None):
+        """
+        Return a duplicate of this BoundMenuCommand augmented with a QWebView.  
+        The methods :func:`htmlJavascript`, :func:`htmlIcons` and :func:`htmlButtons` can 
+        be used to construct html which calls back to this view.
+        
+        The loadFinished signal of the passed QWebView is connected and the 
+        QWebFrame method addToJavaScriptWindowObject is called to associated 
+        this object with name in the javascript name space.
+        
+        :param webView:  the webview which is associated with the returned object
+        :param javaScriptName:  a name for the returned object in the 
+            javascript namespace.  If None, a unique name is auto-generated.
+        """
+        if javaScriptName is None:
+            import uuid
+            javaScriptName = "bound_command_{0}".format(uuid.uuid1().hex)
+        return BoundCommandMenu(self.target, self.menu, webView=webView, javaScriptName=javaScriptName, objectConverter=objectConverter)
+
     def withObject(self, obj, buttonBox=None, objectConverter=None):
         """
         Return a duplicate of this BoundMenuCommand augmented with a view.  The 
@@ -433,6 +506,8 @@ class BoundCommandMenu(QtCore.QObject):
             action = cmd.action(actionParent)
             current = BoundCommandMenuItem(cmd, action, objectConverter = self.objectConverter, binding = weakref.proxy(self))
             self.action_bindings.append(current)
+            if action is None:
+                continue
             if cmd.default:
                 self.default_binding = current
 
@@ -441,12 +516,15 @@ class BoundCommandMenu(QtCore.QObject):
                 button = self.buttonBox.addButton(cmd.descr, QtGui.QDialogButtonBox.ActionRole)
                 self.button_bindings.append(BoundCommandMenuItem(cmd, button, objectConverter = self.objectConverter, binding = weakref.proxy(self)))
 
-        if self.default_binding and self.bindDefault:
-            self.view.activated.connect(self.default_binding.activatedProxy)
+        if self.default_binding:
+            if self.bindDefault is True or self.bindDefault == "activate":
+                self.view.activated.connect(self.default_binding.activatedProxy)
+            elif self.bindDefault == "double":
+                self.view.doubleClicked.connect(self.default_binding.activatedProxy)
 
         if self.view is not None:
             for cb in self.action_bindings:
-                if cb.cmd.viewRelated:
+                if cb.cmd.viewRelated and cb.action is not None:
                     self.view.addAction(cb.action)
 
     def fillMenu(self, menu):
@@ -467,6 +545,52 @@ class BoundCommandMenu(QtCore.QObject):
         for command in self.action_bindings:
             toolbar.addAction(command.action)
 
+    def htmlButtons(self, obj):
+        """
+        Return a string with html button definitions which call the trigger slot.
+        
+        :param javaScriptName:  the name given when passing this binding object to addToJavaScriptWindowObject.
+        """
+        if not hasattr(self, "action_bindings"):
+            self.resetActions()
+        fstring = """<input type="button" id="evalbutton" value="{descr}" onclick="{callback}" />"""
+        btns = [fstring.format(descr=binding.cmd.descr, callback=self.htmlJavascript(binding.cmd.name(), obj)) for binding in self.action_bindings]
+        return ' '.join(btns)
+
+    def htmlIcons(self, obj):
+        """
+        Return a string with html image definitions which call the trigger slot.
+        
+        :param javaScriptName:  the name given when passing this binding object to addToJavaScriptWindowObject.
+        """
+        if not hasattr(self, "action_bindings"):
+            self.resetActions()
+        fstring = """<img src="{icon}" alt="{descr}" onclick="{callback}" />"""
+        btns = [fstring.format(descr=binding.cmd.descr, icon=binding.cmd.iconFile, callback=self.htmlJavascript(binding.cmd.name(), obj)) for binding in self.action_bindings]
+        return ' '.join(btns)
+
+    def htmlJavascript(self, cmd, obj):
+        return "{jsn}.objectActivated('{name}', '{id}')".format(jsn=self.javaScriptName, name=cmd, id=obj)
+
+    def addJavascriptObjects(self):
+        self.webView.page().currentFrame().addToJavaScriptWindowObject(self.javaScriptName, self)
+
+    @Slot(str, str)
+    def objectActivated(self, cmd, obj):
+        if not hasattr(self, "action_bindings"):
+            self.resetActions()
+        for binding in self.action_bindings:
+            if binding.cmd.name() == cmd:
+                binding.objectActivatedProxy(obj, self.objectConverter)
+
+    @Slot(str)
+    def trigger(self, cmd):
+        if not hasattr(self, "action_bindings"):
+            self.resetActions()
+        for binding in self.action_bindings:
+            if binding.cmd.name() == cmd:
+                binding.triggeredProxy()
+
     def __len__(self):
         return len(self.menu.commands)
 
