@@ -5,7 +5,7 @@ from auth import LoginDialog, auth_dialog
 from multiauth import MultiAuthDialog, multi_auth_dialog
 
 from objectforms import MapperDialog, BoundDialog, BoundDialogObject, \
-                        messaged_commit, exception_message_box, SessionMessageBox
+                        messaged_commit, exception_message_box, appMessage, SessionMessageBox
 
 try:
     from PyQt4 import QtDeclarative
