@@ -48,6 +48,12 @@ class InputYoke(object):
         widget.setWhatsThis(ClassAttributeWhatsThis(getattr(self.mapper.cls, self.attr)))
         widget.setProperty("BoundClassField", "{0}.{1}".format(self.mapper.cls.__name__, self.attr))
 
+    def focus(self):
+        self.widget.setFocus(QtCore.Qt.OtherFocusReason)
+
+    def mainWidget(self):
+        return self.widget
+
     def Bind(self):
         """
         Data initialization.
@@ -106,6 +112,16 @@ class LineYoke(InputYoke):
         else:
             raise ValueError("The widget {0} is not supported by a {1}".format(self.widget, self.__class__.__name__))
 
+class PasswordYoke(LineYoke):
+    def Factory(self):
+        LineYoke.Factory(self)
+        self.widget.setEchoMode(QtGui.QLineEdit.Password)
+        return self.widget
+
+    def AdoptWidget(self, widget):
+        LineYoke.AdoptWidget(self, widget)
+        self.widget.setEchoMode(QtGui.QLineEdit.Password)
+
 class BlankIntValidator(QtGui.QIntValidator):
     def validate(self,input,pos):
         if input == "":
@@ -242,6 +258,7 @@ class DateYoke(LineYoke):
         mapper.reverse_yoke(attr,self)
 
     def Factory(self):
+        from widgets import PBDateEdit
         self.widget = PBDateEdit()
         self._baseAdoptWidget(self.widget)
         self.widget.editingFinished.connect(self.Save)
@@ -398,6 +415,10 @@ def yokes_dict():
     d["text"] = TextYoke
     d["formatted"] = FormattedYoke
     d["line"] = LineYoke
+    d["password"] = PasswordYoke
+
+    from widgets import StreetAddressYoke
+    d["address"] = StreetAddressYoke
 
     import foreign_key
     d["foreign_key"] = foreign_key.ForeignKeyEditYoke
